<?php

/**
 * @package Vesthelm Engine
 * @version 1.0.0
 * @copyright Copyright(c) 2010-2011, Anton Kurbanov
 * @license Open Source. For more details, please visit: http://www.vesthelm.com/license.html
 * @info Package generated specially for http://test.ir!
 */
defined('V_IN_ACP') or die('Powered by <a href="http://www.vesthelm.com/">Vesthelm EE</a>');

class Addon extends Vesthelm {

    /**
     * Xml tag
     *
     * @var mixed
     */
    var $xmlTag;
    /**
     * Xml path
     *
     * @var mixed
     */
    var $xmlPath;
    /**
     * Xml attributes
     *
     * @var mixed
     */
    var $xmlAttributes;
    /**
     * Xml level
     *
     * @var int
     */
    var $xmlLevel = 0;
    /**
     * Addon
     *
     * @var string
     */
    var $addon;
    /**
     * Data
     *
     * @var string
     */
    var $data;
    /**
     * Addon action
     *
     * @var string
     */
    var $action;
    /**
     * Addon title
     *
     * @var string
     */
    var $title;
    /**
     * Addon description
     *
     * @var string
     */
    var $description;
    /**
     * Addon author
     *
     * @var string
     */
    var $author;
    /**
     * Addon version
     *
     * @var string
     */
    var $version;
    /**
     * Created date
     *
     * @var string
     */
    var $date;
    /**
     * Configurations
     *
     * @var array
     */
    var $configurations;
    /**
     * Phrases
     *
     * @var array
     */
    var $phrases;
    /**
     * Links
     *
     * @var array
     */
    var $links;
    /**
     * Realms
     *
     * @var array
     */
    var $realms;
    /**
     * Hooks
     *
     * @var array
     */
    var $hooks;
    /**
     * MySQL queries
     *
     * @var array
     */
    var $sqls;
    /**
     * Uninstall MySQL queries
     *
     * @var array
     */
    var $uninstall_sqls;
    /**
     * PHP code
     *
     * @var array
     */
    var $phps;
    /**
     * Uninstall PHP code
     *
     * @var array
     */
    var $uninstall_phps;
    /**
     * Addons table
     *
     * @var string
     */
    var $vTable = 'addons';

    /**
     * Get available addons
     *
     * @return array Available addons
     */
    function getAvailableAddons() {
        $dir = V_ADDONS;
        $key = 0;
        $addons = array();
        $installedAddons = $this->nametovalue('', "`name`,`version`");

        if (is_dir($dir)) {
            if ($dh = opendir($dir)) {
                while (($file = readdir($dh)) !== false) {
                    if (strpos($file, '.') === false) {
                        if (!$this->parseAddon($file))
                            continue;

                        $installed = array_key_exists($file, $installedAddons);
                        $upgrading = $installed ? version_compare($installedAddons[$file], $this->version, "<") : false;

                        if (!$installed || $upgrading) {
                            $addons[$key]['name'] = $this->addon;
                            $addons[$key]['title'] = $this->title;
                            $addons[$key]['description'] = $this->description;
                            $addons[$key]['author'] = $this->author;
                            $addons[$key]['version'] = $this->version;
                            $addons[$key]['action'] = $upgrading ? 'upgrade' : 'install';
                            $key++;
                        }
                    }
                }
                closedir($dh);
            }
        }

        return $addons;
    }

    /**
     * Get installed addons
     *
     * @param int $start
     * @param int $limit
     * @param string $sort_by Sort by field
     * @param string $sort_dir Sort direction
     * @param array $search_in_fields Search in fields
     * @param string $search_value Search value
     * @param string $status Status
     * @return array Installed addons
     */
    function getInstaledAddons($start = 0, $limit = 0, $sort_by = '', $sort_dir = '', $search_in_fields = array(), $search_value = '', $status = '') {
        $search_in_fields = array_map(array($this, "escape"), $search_in_fields);
        $search_value = $this->escape($search_value);
        $status = $this->escape($status);

        $where = !empty($search_in_fields[0]) ? "CONCAT(`" . implode("`,' ',`", $search_in_fields) . "`) LIKE '%{$search_value}%'" : '';
        $where .= ( !empty($search_in_fields[0]) && !empty($status)) ? " AND " : '';
        $where .= ! empty($status) ? "`status` = '{$status}'" : '';

        return $this->all('', '*', $where, $start, $limit, false, true, $sort_by, $sort_dir);
    }

    /**
     * Updates addon(s) status
     *
     * @param string $status Status
     * @param array $addon Addons
     * @return bool
     */
    function updateStatus($status, $addon) {
        $result = false;
        $status = $this->escape($status);

        if (!empty($status) && in_array($status, array("active", "inactive"))) {
            $addon = array_map(array($this, "escape"), $addon);

            $result = $this->update('', array('status' => $status), "`name` IN ('" . implode("','", $addon) . "')");

            if ($result)
                $result = $this->update('hooks', array('status' => $status), "`addon` IN ('" . implode("','", $addon) . "')");
        }

        return $result;
    }

    /**
     * Do addon action(install, upgrade, uninstall)
     *
     * @param string $addon Addon
     * @param string $action Addon action
     * @return bool
     */
    function doAction($addon, $action) {
        $addon = $this->escape($addon);

        if (empty($addon))
            $this->debugMessage("Empty addon name...");

        switch ($action) {
            case 'install':
            case 'upgrade':
            case 'uninstall':
                if (!$this->parseAddon($addon))
                    return false;

                $this->action = $action;

                $this->configurations();
                $this->phrases();
                $this->links();
                $this->realms();
                $this->hooks();
                $this->sqls();
                $this->phps();

                $this->callHook('doActionAddonIntegration');

                $this->addons();

                break;

            default:
                $this->debugMessage("Unknown addon action...");
        }

        return true;
    }

    /**
     * Parse addon xml file
     *
     * @param string $addon Addon
     * @return bool
     */
    function parseAddon($addon) {
        $result = false;
        $addonPath = V_ADDONS . $addon . V_DS . 'installation' . V_DS;
        $installFile = 'installation.xml';
        $installFilePath = $addonPath . $installFile;

        if (file_exists($installFilePath)) {
            require_once(V_LIBS . "xml" . V_DS . "xml_saxy_parser.php");

            $parser = new SAXY_Parser();
            $parser->xml_set_element_handler(array(&$this, "startElement"), array(&$this, "endElement"));
            $parser->xml_set_character_data_handler(array(&$this, "charData"));

            $xmlContent = @file_get_contents($installFilePath);

            if (!empty($xmlContent)) {
                $this->addon = $addon;

                $parser->parse($xmlContent);

                $result = true;
            }
        }

        return $result;
    }

    /**
     * Configurations actions(install, upgrade, uninstall)
     *
     * @return bool
     */
    function configurations() {
        $this->setTable('configurations');

        if ($this->configurations) {
            $maxorder = $this->one('', "MAX(`order`)");
            $maxorder = empty($maxorder) ? 1 : ($maxorder + 1);

            switch ($this->action) {
                case 'install':
                    foreach ($this->configurations as $key => $configuration) {
                        $this->configurations[$key]['order'] = $maxorder++;
                    }

                    $this->insert('', $this->configurations);
                    break;

                case 'upgrade':
                    $cfgs = array();

                    foreach ($this->configurations as $key => $configuration) {
                        if ($this->exists('', "`name` = '{$configuration['name']}' AND `addon` = '{$configuration['addon']}'")) {
                            $this->update('', $configuration, "`name` = '{$configuration['name']}' AND `addon` = '{$configuration['addon']}'");
                        } else {
                            $configuration['order'] = $maxorder++;

                            $cfgs[] = $configuration;
                        }
                    }

                    if (!empty($cfgs))
                        $this->insert('', $cfgs);

                    break;

                case 'uninstall':
                    $this->delete('', "`addon` = '" . $this->addon . "'");
                    break;
            }

            $Config = &Config::getInstance();
            $Config->createConfigCacheFiles();
        }

        $this->resetTable();

        return true;
    }

    /**
     * Phrases actions(install, upgrade, uninstall)
     *
     * @return bool
     */
    function phrases() {
        $this->setTable('languages_phrases');

        if ($this->phrases) {
            switch ($this->action) {
                case 'install':
                    foreach ($this->vLanguages as $code => $language) {
                        foreach ($this->phrases as $key => $phrase) {
                            $this->phrases[$key]['code'] = $code;
                        }

                        $this->insert('', $this->phrases);
                    }
                    break;

                case 'upgrade':
                    foreach ($this->vLanguages as $code => $language) {
                        $phrs = array();

                        foreach ($this->phrases as $key => $phrase) {
                            if ($this->exists('', "`name` = '{$phrase['name']}' AND `side` = '{$phrase['side']}' AND `addon` = '{$phrase['addon']}' AND `code` = '{$code}'")) {
                                $this->update('', $phrase, "`name` = '{$phrase['name']}' AND `side` = '{$phrase['side']}' AND `addon` = '{$phrase['addon']}' AND `code` = '{$code}'");
                            } else {
                                $phrase['code'] = $code;

                                $phrs[] = $phrase;
                            }
                        }

                        if (!empty($phrs))
                            $this->insert('', $phrs);
                    }

                    break;

                case 'uninstall':
                    $this->delete('', "`addon` = '" . $this->addon . "'");
                    break;
            }

            $this->createAllPhrasesCacheFiles();
        }
        $this->resetTable();

        return true;
    }

    /**
     * Links actions(install, upgrade, uninstall)
     *
     * @return bool
     */
    function links() {
        $this->setTable('links');

        if ($this->links) {
            $maxorder = $this->one('', "MAX(`order`)");
            $maxorder = empty($maxorder) ? 1 : ($maxorder + 1);

            switch ($this->action) {
                case 'install':
                    $wlinks = array();

                    foreach ($this->links as $key => $link) {
                        if (!empty($link['parent_title']) && !empty($link['parent_group_name'])) {
                            $wlinks[] = $link;

                            unset($this->links[$key]);
                            continue;
                        }

                        unset($this->links[$key]['parent_title'], $this->links[$key]['parent_group_name']);
                        $this->links[$key]['order'] = $maxorder++;
                    }

                    $this->insert('', $this->links);

                    if (!empty($wlinks)) {
                        foreach ($wlinks as $key => $wlink) {
                            $wlinks[$key]['order'] = $maxorder++;
                            $wlinks[$key]['parent_id'] = (int) $this->one('', '`id`', "`title` = '" . $wlink['parent_title'] . "' AND `group_name` = '" . $wlink['parent_group_name'] . "'");

                            unset($wlinks[$key]['parent_title'], $wlinks[$key]['parent_group_name']);
                        }

                        $this->insert('', $wlinks);
                    }

                    break;

                case 'upgrade':
                    $lnks = array();

                    foreach ($this->links as $key => $link) {
                        unset($link['parent_title'], $link['parent_group_name']);

                        if ($this->exists('', "`title` = '{$link['title']}' AND `group_name` = '{$link['group_name']}' AND `addon` = '{$link['addon']}'")) {
                            $this->update('', $link, "`title` = '{$link['title']}' AND `group_name` = '{$link['group_name']}' AND `addon` = '{$link['addon']}'");
                        } else {
                            $link['order'] = $maxorder++;

                            $lnks[] = $link;
                        }
                    }

                    if (!empty($lnks))
                        $this->insert('', $lnks);
                    break;

                case 'uninstall':
                    $this->delete('', "`addon` = '" . $this->addon . "'");
                    break;
            }
        }

        $this->resetTable();

        return true;
    }

    /**
     * Realms actions(install, upgrade, uninstall)
     *
     * @return bool
     */
    function realms() {
        $this->setTable('realms');

        if ($this->realms) {
            switch ($this->action) {
                case 'install':
                    $this->insert('', $this->realms);
                    break;

                case 'upgrade':
                    $rlms = array();

                    foreach ($this->realms as $key => $realm) {
                        if ($this->exists('', "`file` = '{$realm['file']}' AND `addon` = '{$realm['addon']}'")) {
                            $this->update('', $realm, "`file` = '{$realm['name']}' AND `addon` = '{$realm['addon']}'");
                        } else {
                            $rlms[] = $realm;
                        }
                    }

                    if (!empty($rlms))
                        $this->insert('', $rlms);

                    break;

                case 'uninstall':
                    $this->delete('', "`addon` = '" . $this->addon . "'");
                    break;
            }
        }

        $this->resetTable();

        return true;
    }

    /**
     * Hooks actions(install, upgrade, uninstall)
     *
     * @return bool
     */
    function hooks() {
        $this->setTable('hooks');

        if ($this->hooks) {
            $maxorder = $this->one('', "MAX(`order`)");
            $maxorder = empty($maxorder) ? 1 : ($maxorder + 1);

            switch ($this->action) {
                case 'install':
                    foreach ($this->hooks as $key => $hook) {
                        $this->hooks[$key]['order'] = $maxorder++;
                    }

                    $result = $this->insert('', $this->hooks);

                    if ($result && 'require' == $this->vConfig['hooks_run_type']) {
                        foreach ($this->hooks as $key => $hook) {
                            if ('php' == $hook['code']) {
                                $this->vWriter->writeToFile(trim($hook['name']) . '_' . $hook['u_id'] . '.php', '<?php ' . trim($hook['value']) . ' ?>');
                            }
                        }
                    }

                    break;

                case 'upgrade':
                    $hks = array();

                    foreach ($this->hooks as $key => $hook) {
                        if ($this->exists('', "`name` = '{$hook['name']}' AND `addon` = '{$hook['addon']}'")) {
                            $result = $this->update('', $hook, "`name` = '{$hook['name']}' AND `addon` = '{$hook['addon']}'");

                            if ($result && 'require' == $this->vConfig['hooks_run_type']) {
                                if ('php' == $hook['code']) {
                                    $this->vWriter->writeToFile(trim($hook['name']) . '_' . $hook['u_id'] . '.php', '<?php ' . trim($hook['value']) . ' ?>');
                                }
                            }
                        } else {
                            $hook['order'] = $maxorder++;

                            $hks[] = $hook;
                        }
                    }

                    if ($hks) {
                        $result = $this->insert('', $hks);

                        if ($result && 'require' == $this->vConfig['hooks_run_type']) {
                            foreach ($hks as $key => $hook) {
                                if ('php' == $hook['code']) {
                                    $this->vWriter->writeToFile(trim($hook['name']) . '_' . $hook['u_id'] . '.php', '<?php ' . trim($hook['value']) . ' ?>');
                                }
                            }
                        }
                    }
                    break;

                case 'uninstall':
                    $hooks = $this->all('', '`id`,`name`,`u_id`', "`addon` = '" . $this->escape($this->addon) . "'");

                    if ('require' == $this->vConfig['hooks_run_type']) {
                        if ($hooks) {
                            foreach ($hooks as $hook) {
                                $this->vWriter->removeFile(trim($hook['name']) . '_' . $hook['u_id'] . '.php'); //delete hook file

                                foreach ($this->vHooks as $k => $rhook) {
                                    if ($rhook['id'] == $hook['id']) {
                                        unset($this->vHooks[$k]);
                                    }
                                }
                            }
                        }
                    }

                    $this->delete('', "`addon` = '" . $this->escape($this->addon) . "'");

                    break;
            }
        }

        $this->resetTable();

        return true;
    }

    /**
     * Sqls actions(install, upgrade, uninstall)
     *
     * @return bool
     */
    function sqls() {
        if (!empty($this->sqls[$this->action])) {
            foreach ($this->sqls[$this->action] as $sql) {
                switch ($this->action) {
                    case 'install':
                    case 'uninstall':
                    case 'upgrade':
                        $this->query($sql);
                        break;
                }
            }
        }

        return true;
    }

    /**
     * Phps actions(install, upgrade, uninstall)
     *
     * @return bool
     */
    function phps() {
        if (!empty($this->phps[$this->action])) {
            foreach ($this->phps[$this->action] as $php) {
                switch ($this->action) {
                    case 'install':
                    case 'uninstall':
                    case 'upgrade':
                        eval($php);
                        break;
                }
            }
        }

        return true;
    }

    /**
     * Addons actions(install, upgrade, uninstall)
     *
     * @return bool
     */
    function addons() {
        if ($this->addon) {
            switch ($this->action) {
                case 'install':
                    $addon = array(
                        'name' => $this->addon,
                        'title' => $this->title,
                        'description' => $this->description,
                        'author' => $this->author,
                        'status' => 'active',
                        'date' => $this->date,
                        'version' => $this->version,
                        'uninstall_sqls' => !empty($this->uninstall_sqls) ? serialize($this->uninstall_sqls) : '',
                        'uninstall_phps' => !empty($this->uninstall_phps) ? implode("\r\n", $this->uninstall_phps) : ''
                    );

                    $this->insert('', $addon);
                    break;

                case 'upgrade':
                    $addon = array(
                        'name' => $this->addon,
                        'title' => $this->title,
                        'description' => $this->description,
                        'author' => $this->author,
                        'date' => $this->date,
                        'version' => $this->version
                    );

                    $this->update('', $addon, "`name` = '" . $this->addon . "'");
                    break;

                case 'uninstall':
                    $custom_actions = $this->row('', "`uninstall_phps`,`uninstall_sqls`", "`name` = '" . $this->addon . "'");

                    if ($custom_actions) {
                        if (!empty($custom_actions['uninstall_sqls'])) {
                            $a_sql_queries = unserialize($custom_actions['uninstall_sqls']);

                            if (is_array($a_sql_queries) && !empty($a_sql_queries)) {
                                foreach ($a_sql_queries as $key => $a_sql_query) {
                                    $this->query(trim($a_sql_query));
                                }
                            }
                        }

                        if (!empty($custom_actions['uninstall_phps'])) {
                            eval($custom_actions['uninstall_phps']);
                        }
                    }

                    $this->delete('', "`name` = '" . $this->addon . "'");
                    break;
            }
        }

        $this->resetTable();

        return true;
    }

    /**
     * Start element
     *
     * @param mixed $parser Xml parser
     * @param mixed $name
     * @param mixed $attributes Xml attributes
     * @return int Xml level
     */
    function startElement($parser, $name, $attributes) {
        $this->xmlLevel++;
        $this->xmlTag = $name;
        $this->xmlPath[] = $name;
        $this->xmlAttributes = $attributes;

        return $this->xmlLevel;
    }

    /**
     * End element
     *
     * @param mixed $parser Xml parser
     * @param mixed $name
     * @return int Xml level
     */
    function endElement($parser, $name) {
        $this->xmlLevel--;
        array_pop($this->xmlPath);

        return $this->xmlLevel;
    }

    /**
     * Char data
     *
     * @param mixed $parser Xml parser
     * @param mixed $this->data
     * @return int Xml level
     */
    function charData($parser, $data) {
        $this->data = trim($data);

        switch ($this->xmlTag) {
            case 'configuration':
                $this->configurations[] = array(
                    "group_name" => isset($this->xmlAttributes['group_name']) ? $this->xmlAttributes['group_name'] : $this->addon,
                    "name" => $this->xmlAttributes['name'],
                    "value" => $this->data,
                    "values" => isset($this->xmlAttributes['values']) ? $this->xmlAttributes['values'] : '',
                    "hide_in_js" => isset($this->xmlAttributes['hide_in_js']) ? $this->xmlAttributes['hide_in_js'] : 0,
                    "type" => in_array($this->xmlAttributes['type'], array('text', 'htmleditor', 'textarea', 'combo', 'radio', 'number', 'divider', 'hidden')) ? $this->xmlAttributes['type'] : 'text',
                    "width" => isset($this->xmlAttributes['width']) ? $this->xmlAttributes['width'] : 200,
                    "height" => isset($this->xmlAttributes['height']) ? $this->xmlAttributes['height'] : 200,
                    "addon" => $this->addon
                );
                break;

            case 'phrase':
                $this->phrases[] = array(
                    "name" => $this->xmlAttributes['name'],
                    "value" => $this->data,
                    "side" => isset($this->xmlAttributes['side']) ? $this->xmlAttributes['side'] : 'acp',
                    "addon" => $this->addon
                );
                break;

            case 'link':
                $this->links[] = array(
                    "file" => isset($this->xmlAttributes['file']) ? $this->xmlAttributes['file'] : 'index',
                    "action" => isset($this->xmlAttributes['action']) ? $this->xmlAttributes['action'] : '',
                    "params" => isset($this->xmlAttributes['params']) ? $this->xmlAttributes['params'] : '',
                    "title" => $this->xmlAttributes['title'],
                    "group_name" => isset($this->xmlAttributes['group_name']) ? $this->xmlAttributes['group_name'] : $this->addon,
                    "addon" => $this->addon,
                    "parent_title" => isset($this->xmlAttributes['parent_title']) ? $this->xmlAttributes['parent_title'] : '',
                    "parent_group_name" => isset($this->xmlAttributes['parent_group_name']) ? $this->xmlAttributes['parent_group_name'] : ''
                );

                $this->realms[] = array(
                    "file" => isset($this->xmlAttributes['file']) ? $this->xmlAttributes['file'] : '',
                    "action" => isset($this->xmlAttributes['action']) ? $this->xmlAttributes['action'] : '',
                    "phrase_name" => $this->xmlAttributes['title'],
                    "side" => 'acp',
                    "addon" => $this->addon
                );

                break;

            case 'realm':
                $this->realms[] = array(
                    "file" => isset($this->xmlAttributes['file']) ? $this->xmlAttributes['file'] : '',
                    "action" => isset($this->xmlAttributes['action']) ? $this->xmlAttributes['action'] : '',
                    "phrase_name" => $this->xmlAttributes['phrase_name'],
                    "side" => in_array($this->xmlAttributes['side'], array('acp', 'frontend')) ? $this->xmlAttributes['side'] : 'frontend',
                    "addon" => $this->addon
                );
                break;

            case 'hook':
                $this->hooks[] = array(
                    "name" => $this->xmlAttributes['name'],
                    "value" => $this->data,
                    "code" => isset($this->xmlAttributes['code']) ? $this->xmlAttributes['code'] : 'php',
                    "status" => isset($this->xmlAttributes['status']) ? $this->xmlAttributes['status'] : 'active',
                    "addon" => $this->addon,
                    "side" => isset($this->xmlAttributes['side']) ? $this->xmlAttributes['side'] : 'frontend',
                    "u_id" => Utils::newToken()
                );
                break;

            case 'sql':
                if (in_array('install', $this->xmlPath))
                    $this->sqls['install'][] = str_replace('{vprefix}', $this->vPrefix, $this->data);
                elseif (in_array('uninstall', $this->xmlPath))
                    $this->sqls['uninstall'][] = str_replace('{vprefix}', $this->vPrefix, $this->data);
                elseif (in_array('upgrade', $this->xmlPath))
                    $this->sqls['upgrade'][] = str_replace('{vprefix}', $this->vPrefix, $this->data);
                break;

            case 'php':
                if (in_array('install', $this->xmlPath))
                    $this->phps['install'][] = $this->data;
                elseif (in_array('uninstall', $this->xmlPath))
                    $this->phps['uninstall'][] = $this->data;
                elseif (in_array('upgrade', $this->xmlPath))
                    $this->phps['upgrade'][] = $this->data;
                break;

            case 'title':
                $this->title = $this->data;
                break;

            case 'description':
                $this->description = $this->data;
                break;

            case 'author':
                $this->author = $this->data;
                break;

            case 'version':
                $this->version = $this->data;
                break;

            case 'date':
                $this->date = $this->data;
                break;
        }

        //addons integration
        if (in_array('integrations', $this->xmlPath)) {
            $this->callHook('CharDataAddonIntegration');
        }

        return $this->xmlLevel;
    }

}

?>