<?php

/**
 * @package Vesthelm Engine
 * @version 1.0.0
 * @copyright Copyright(c) 2010-2011, Anton Kurbanov
 * @license Open Source. For more details, please visit: http://www.vesthelm.com/license.html
 * @info Package generated specially for http://test.ir!
 */
defined('V_MAIN') or die('Powered by <a href="http://www.vesthelm.com/">Vesthelm EE</a>');

class Config extends Database {

    /**
     * Writer
     *
     * @var object
     */
    var $vWriter;
    /**
     * Config
     *
     * @var array
     */
    var $vConf;
    /**
     * Configuration table
     *
     * @var string
     */
    var $vTable = 'configurations';

    /**
     * Configuration constructor
     */
    function Config() {
        $this->vConfig['vdb_host'] = V_DB_HOST;
        $this->vConfig['vdb_user'] = V_DB_USER;
        $this->vConfig['vdb_password'] = V_DB_PASSWORD;
        $this->vConfig['vdb_name'] = V_DB_NAME;
        $this->vPrefix = V_PREFIX;
        $this->connect();

        $this->vWriter = new Writer();

        $this->vConf = $this->vWriter->getContents('vconfig', true);

        $this->checkConfigCacheFiles();

        $this->vConfig['vdb_host'] = '';
        $this->vConfig['vdb_user'] = '';
        $this->vConfig['vdb_password'] = '';
        $this->vConfig['vdb_name'] = '';
    }

    /**
     * Get configuration value
     *
     * @param string $name Configuration name
     * @param bool $unserialize Unserialize configuration
     * @param bool $fromdatabase Get configuration directly from database
     * @return mixed Configuration value
     */
    function get($name, $unserialize = false, $fromdatabase = false) {
        if (!empty($this->vConf) && !$fromdatabase) {
            $value = ($unserialize) ? unserialize($this->vConf[$name]) : $this->vConf[$name];
        } else {
            $value = $this->one('', '`value`', "`name` = '{$name}'");

            if ($unserialize) {
                $value = unserialize($value[0]);
            }
        }

        return $value;
    }

    /**
     * Set configuration value
     *
     * @param string $name Configuration name
     * @param mixed $value Configuration value
     * @param bool $serialize Serialize configuration
     * @param bool $todatabase Insert configuration directly to database
     * @param bool $update_cache Update configuration cache
     * @return int Inserted id or bool
     */
    function set($name, $value = '', $serialize = false, $todatabase = false, $update_cache = false) {
        $result = true;
        $value = ($serialize) ? serialize($value) : $value;

        $this->vConf[$name] = $value;

        if ($todatabase) {
            if (!$this->exists('', "`name`= '{$name}'")) {
                $result = $this->insert('', array('name' => $name, 'value' => $value));
            } else {
                $result = $this->update('', array('name' => $name, 'value' => $value), "`name`= '{$name}'");
            }
        }

        if ($update_cache) {
            $this->createConfigCacheFiles();
        }

        return $result;
    }

    /**
     * Check configuration cache files
     * 
     * @return bool
     */
    function checkConfigCacheFiles() {
        $pfile = $this->vWriter->filePath . 'vconfig';

        if (file_exists($pfile) && file_exists($pfile . '.js') && !empty($this->vConf)) {
            if (isset($this->vConf['configurations_cache_time']) && (int) $this->vConf['configurations_cache_time'] > 0) {
                $time = (int) $this->vConfig['configurations_cache_time'];
            } else {
                $time = time() - (14 * 24 * 60 * 60); //14 days; 24 hours; 60 mins; 60secs
            }

            if (filemtime($pfile) < $time || filemtime($pfile . '.js') < $time) {
                $this->createConfigCacheFiles();
            }
        } else {
            $this->createConfigCacheFiles();
        }

        return true;
    }

    /**
     * Create configuration cache files
     * 
     * @return bool
     */
    function createConfigCacheFiles() {
        require_once('Json.php');
        $Json = new Services_JSON();

        $configurations = $this->all('', '`name`,`value`,`hide_in_js`', "`type` <> 'divider'");

        if ($configurations) {
            foreach ($configurations as $configuration) {
                $this->vConf[$configuration['name']] = $configuration['value'];
            }
        }

        $lvals = $this->all('languages', '`code`,`title`,`charset`,`dir`');

        $cfgs = array();

        if ($lvals) {
            foreach ($lvals as $value) {
                $cfgs['languages'][$value['code']] = $this->vConf['languages'][$value['code']] = $value['title'];
                $cfgs['charsets'][$value['code']] = $this->vConf['charsets'][$value['code']] = $value['charset'];
                $cfgs['text_dirs'][$value['code']] = $this->vConf['text_dirs'][$value['code']] = $value['dir'];
            }
        }

        if ($configurations) {
            foreach ($configurations as $configuration) {
                if (!$configuration['hide_in_js']) {
                    $cfgs[$configuration['name']] = $configuration['value'];
                }
            }
        }

        $cfgs['applications_mod_rewrite_prefix'] = V_APPLICATIONS_MOD_REWRITE_PREFIX; //to js
        $cfgs['num_languages'] = count($lvals);

        $this->vWriter->writeToFile('vconfig', $this->vConf);
        $this->vWriter->writeToFile('vconfig.js', "vconfig = " . $Json->encode($cfgs) . ";");

        return true;
    }

    /**
     * Get configuration object
     *
     * @return object Configuration object
     */
    function &getInstance() {
        static $instance = null;

        if (null === $instance) {
            $instance = new Config();
        }

        return $instance;
    }

}

?>