<?php

/**
 * @package Vesthelm Engine
 * @version 1.0.0
 * @copyright Copyright(c) 2010-2011, Anton Kurbanov
 * @license Open Source. For more details, please visit: http://www.vesthelm.com/license.html
 * @info Package generated specially for http://test.ir!
 */
defined('V_MAIN') or die('Powered by <a href="http://www.vesthelm.com/">Vesthelm EE</a>');

class Database {

    /**
     * Error message from query
     *
     * @var string
     */
    var $error = '';
    /**
     * Numerical value of the error message from query
     *
     * @var mixed
     */
    var $errno = 0;
    /**
     * Number of rows
     *
     * @var mixed
     */
    var $num_rows = 0;
    /**
     * Affected rows by last query
     *
     * @var mixed
     */
    var $affected_rows = 0;
    /**
     * Link identifier
     *
     * @var mixed
     */
    var $link_id = 0;
    /**
     * Query identifier
     *
     * @var mixed
     */
    var $query_id = 0;
    /**
     * Useful information
     *
     * @var array
     */
    var $info;
    /**
     * Configuration
     *
     * @var array
     */
    var $vConfig;
    /**
     * Table
     *
     * @var string
     */
    var $vTable = '';
    /**
     * Backup table
     *
     * @var string
     */
    var $tableBackup = '';
    /**
     * Backup prefix
     *
     * @var string
     */
    var $prefixBackup = '';
    var $ops = array('AND', 'OR', 'LIKE');

    /**
     * Connects to database
     *
     * @param bool $new_link Open new link
     * @return mixed MySQL link identifier on success or FALSE on failure.
     */
    function connect($new_link = false) {
        $this->link_id = @mysql_connect($this->vConfig['vdb_host'], $this->vConfig['vdb_user'], $this->vConfig['vdb_password'], $new_link);

        if (!$this->link_id) {
            $this->debugMessage("Could not connect to server: {$this->vConfig['vdb_host']}.");
        }

        if (!mysql_select_db($this->vConfig['vdb_name'], $this->link_id)) {
            $this->debugMessage("Could not open database: {$this->vConfig['vdb_name']}.");
        }

        mysql_query('SET NAMES "utf8"', $this->link_id);

        return $this->link_id;
    }

    /**
     * Table that will be used in queries
     *
     * @param string $table Table
     * @param string $new_prefix New table prefix
     * @return string Previous table
     */
    function setTable($table, $new_prefix = '') {
        $this->tableBackup = $this->vTable;

        $this->vTable = $table;

        if (!empty($new_prefix)) {
            $this->prefixBackup = $this->vPrefix;
            $this->vPrefix = $new_prefix;
        }

        return $this->tableBackup;
    }

    /**
     * Restore table that was changed by setTable()
     *
     * @return string Previous table
     */
    function resetTable() {
        if (empty($this->tableBackup)) {
            return false;
        }

        $table = $this->vTable;
        $this->vTable = $this->tableBackup;
        $this->tableBackup = '';

        if (!empty($this->prefixBackup)) {
            $this->vPrefix = $this->prefixBackup;
            $this->prefixBackup = '';
        }

        return $table;
    }

    /**
     * Escapes special characters in a string for use in an SQL query
     *
     * @return mixed Escaped string or FALSE on error
     */
    function escape($string) {
        return mysql_real_escape_string($string/* , $this->link_id */);
    }

    /**
     * Send MySQL query
     *
     * @param string $sql SQL query
     * @return mixed
     */
    function query($sql) {
        if (V_DEBUG > 0) {
            list($usec, $seconds) = explode(' ', microtime());
            $timer = (float) $usec + (float) $seconds;
            $bt = debug_backtrace();
        }

        if (!is_resource($this->link_id)) {
            $this->connect();
        }

        $this->query_id = mysql_query($sql, $this->link_id);

        if (V_DEBUG > 0) {
            list($usec, $seconds) = explode(' ', microtime());
            $stop = (float) $usec + (float) $seconds;
            $time = $stop - $timer;

            $this->info[] = array('class' => isset($bt[2]['class']) ? $bt[2]['class'] : '',
                'function' => isset($bt[2]['function']) ? $bt[2]['function'] : '',
                'query' => $sql,
                'file' => isset($bt[2]['file']) ? $bt[2]['file'] : '',
                'time' => $time
            );
        }

        if (!$this->query_id) {
            $this->debugMessage();
        }

        $this->num_rows = $this->numRows($this->query_id);
        $this->affected_rows = $this->affectedRows($this->link_id);

        return $this->query_id;
    }

    /**
     * Get associative array
     *
     * @param string $table Selects from table
     * @param string $fields Selects 2 fields(`name`,`value`)
     * @param string $where WHERE clause
     * @param int $start LIMIT clause
     * @param int $limit LIMIT clause
     * @return array Associative array where array key is field `name` ie. array('name' => 'value')
     */
    function nametovalue($table = '', $fields = '', $where = '', $start = 0, $limit = 0) {
        $table = $this->table($table);
        $where = $this->where($where);

        $sql = "SELECT {$fields} FROM `{$table}` ";
        $sql .= ! empty($where) ? "WHERE {$where}" : '';
        $sql .= ( $limit > 0) ? " LIMIT " . (int) $start . "," . (int) $limit : '';

        $result = $this->query($sql);
        $out = array();

        while ($row = $this->fetchRow($result)) {
            $out[$row[0]] = $row[1];
        }

        return $out;
    }

    /**
     * Get associative array or enumerated array by sql query
     *
     * @param string $sql SQL query
     * @param bool $num_array Result row as an associative array or as an enumerated array
     * @return array
     */
    function sql($sql, $num_array = false) {
        $result = $this->query($sql);
        $out = array();

        if ($num_array) {
            while ($row = $this->fetchRow($result)) {
                $out[] = $row;
            }
        } else {
            while ($row = $this->fetchAssoc($result)) {
                $out[] = $row;
            }
        }

        return $out;
    }

    /**
     * Get all rows
     *
     * @param string $table Selects from table
     * @param string $fields Selects fields(`field1`,`field2` etc.)
     * @param string $where WHERE clause
     * @param int $start LIMIT clause
     * @param int $limit LIMIT clause
     * @param bool $num_array Result row as an associative array or as an enumerated array
     * @param bool $found_rows Obtain row count then you can return number of rows by foundRows()
     * @param string $sort_by Sort by field
     * @param string $sort_dir Sort direction
     * @param string $group_by Group by field
     * @return array
     */
    function all($table = '', $fields, $where = '', $start = 0, $limit = 0, $num_array = false, $found_rows = false, $sort_by = '', $sort_dir = '', $group_by = '') {
        $table = $this->table($table);
        $where = $this->where($where);
        $fr = $found_rows ? "SQL_CALC_FOUND_ROWS " : '';

        $sql = "SELECT {$fr}{$fields} FROM `{$table}` ";
        $sql .= ! empty($where) ? "WHERE {$where} " : '';
        $sql .= ! empty($group_by) ? "GROUP BY " . $this->escape($group_by) . " " : '';
        $sql .= ! empty($sort_by) ? "ORDER BY " . $this->escape($sort_by) . " " : '';
        $sql .= ! empty($sort_dir) ? $this->escape($sort_dir) . " " : '';
        $sql .= ( $limit > 0) ? "LIMIT " . (int) $start . ", " . (int) $limit : '';

        $result = $this->query($sql);
        $out = array();

        if ($num_array) {
            while ($row = $this->fetchRow($result)) {
                $out[] = $row;
            }
        } else {
            while ($row = $this->fetchAssoc($result)) {
                $out[] = $row;
            }
        }

        return $out;
    }

    /**
     * Get row
     *
     * @param string $table Selects from table
     * @param string $fields Selects fields(`field1`,`field2` etc.)
     * @param string $where WHERE clause
     * @param bool $num_array Result row as an associative array or as an enumerated array
     * @return array
     */
    function row($table = '', $fields, $where = '', $num_array = false) {
        $table = $this->table($table);
        $where = $this->where($where);

        $sql = "SELECT {$fields} FROM `{$table}` ";
        $sql .= ! empty($where) ? "WHERE {$where}" : '';

        $result = $this->query($sql);

        if ($num_array) {
            $out = $this->fetchRow($result);
        } else {
            $out = $this->fetchAssoc($result);
        }

        return $out;
    }

    /**
     * Get field value
     *
     * @param string $table Selects from table
     * @param string $field Selects field(`field`)
     * @param string $where WHERE clause
     * @return mixed
     */
    function one($table = '', $field = '', $where = '') {
        $table = $this->table($table);
        $where = $this->where($where);

        $sql = "SELECT {$field} FROM `{$table}` ";
        $sql .= ! empty($where) ? "WHERE {$where}" : '';

        $result = $this->query($sql);
        $row = $this->fetchRow($result);

        return $row[0];
    }

    /**
     * Update
     *
     * @param string $table Table
     * @param array $data Data (array('field' => 'value'))
     * @param string $where WHERE clause
     * @param array $additional Allow to add sql functions like NOW() etc.
     * @return bool
     */
    function update($table = '', $data = array(), $where = '', $additional = array()) {
        $table = $this->table($table);
        $where = $this->where($where);

        $sql = "UPDATE `{$table}` SET ";

        $set_values = '';

        if (!empty($data)) {
            foreach ($data as $key => $value) {
                $set_values .= "`{$key}` = '" . $this->escape($value) . "',";
            }
        }

        if (!empty($additional)) {
            foreach ($additional as $key => $value) {
                $set_values .= "`{$key}` = " . $this->escape($value) . ",";
            }
        }

        $sql .= substr($set_values, 0, -1);
        $sql .= ! empty($where) ? " WHERE {$where}" : '';

        return $this->query($sql);
    }

    /**
     * Insert
     *
     * @param string $table Table
     * @param array $data Data (array('field' => 'value'))
     * @param string $where WHERE clause
     * @param array $additional Allow to add sql functions like NOW() etc.
     * @return mixed Inserted id or FALSE if no MySQL connection was established
     */
    function insert($table = '', $data = array(), $additional = array()) {
        $table = $this->table($table);
        $fields = '';
        $addit = '';

        if (!empty($data)) {
            if (!isset($data[0])) {
                $data = array($data);
            }

            $fields .= "`" . implode("`,`", array_keys($data[0])) . "`";
        }

        if (!empty($additional)) {
            $c = !empty($fields) ? "," : "";

            $fields .= $c . "`" . implode("`,`", array_keys($additional)) . "`";

            foreach ($additional as $value) {
                $addit .= $c . $this->escape($value) . ",";
            }

            $addit = substr($addit, 0, -1);
        }

        $sql = "INSERT INTO `{$table}` (" . $fields . ") VALUES ";

        $str = '';

        if (!empty($fields)) {
            if ($data) {
                foreach ($data as $value) {
                    $str .= '(';
                    $values = '';

                    foreach ($value as $val) {
                        $values .= "'" . $this->escape($val) . "',";
                    }

                    $values = substr($values, 0, -1);

                    $str .= $values . $addit . '),';
                }
            } else {
                $str .= '(' . $addit . ') ';
            }
        }

        $sql .= substr($str, 0, -1);

        $this->query($sql);

        return $this->insertId($this->link_id);
    }

    /**
     * Delete
     *
     * @param string $table Table
     * @param string $where WHERE clause
     * @return bool
     */
    function delete($table = '', $where = '') {
        if (empty($where)) {
            $this->debugMessage("Empty `where` parameter");
        }

        $table = $this->table($table);
        $where = $this->where($where);

        $sql = "DELETE FROM `{$table}` WHERE {$where}";

        return $this->query($sql);
    }

    /**
     * Check if record exists
     *
     * @param string $table Table
     * @param string $where WHERE clause
     * @return bool
     */
    function exists($table = '', $where = '') {
        $table = $this->table($table);
        $where = $this->where($where);

        $sql = "SELECT 1 FROM `{$table}` WHERE {$where}";

        $result = $this->query($sql);

        return $this->numRows($result) > 0;
    }

    /**
     * Build sql where
     *
     * @param mixed $where Where
     * @return mixed
     */
    function where($where) {
        if (is_array($where)) {
            $arr = array();
            foreach ($where as $key => $value) {
                if (in_array($key, $this->ops)) {
                    $arr[] = $key;
                    continue;
                }

                $arr[] = "`{$key}` = '" . $this->escape($value) . "'";
            }
            $where = implode(" ", $arr);
        }

        return $where;
    }

    /**
     * Check table
     *
     * @param string $table Table
     * @return string
     */
    function table($table) {
        $table = !empty($table) ? $this->vPrefix . $table : $this->vPrefix . $this->vTable;

        return $table;
    }

    /**
     * Get result data
     *
     * @param resource $result Resource result comes from query
     * @param int $row Row number from the result that's being retrieved
     * @param mixed $field Name or offset of the field being retrieved
     * @return mixed Contents of one cell from a MySQL result set or 0
     */
    function result($result, $row = 0, $field = 0) {
        if ($this->numRows($result) > 0) {
            return mysql_result($result, $row, $field);
        } else {
            return 0;
        }
    }

    /**
     * Get number of rows using SQL_CALC_FOUND_ROWS
     *
     * @return int
     */
    function foundRows() {
        $result = $this->query("SELECT FOUND_ROWS()");

        return $this->result($result);
    }

    /**
     * Get number of rows in result
     *
     * @param resource $result Resource result comes from query
     * @return int
     */
    function numRows($result) {
        if (is_resource($result)) {
            return mysql_num_rows($result);
        }

        return 0;
    }

    /**
     * Retrieves the ID generated for an AUTO_INCREMENT column by the previous query (usually INSERT)
     *
     * @param resource $link_id Link identifier
     * @return mixed
     */
    function insertId($link_id) {
        return mysql_insert_id($link_id);
    }

    /**
     * Get number of affected rows in previous MySQL operation
     *
     * @param resource $link_id Link identifier
     * @return mixed
     */
    function affectedRows($link_id) {
        return mysql_affected_rows($link_id);
    }

    /**
     * Count all records in a table
     *
     * @param string $table Table
     * @param string $where WHERE clause
     * @return int
     */
    function count($table = '', $where = '') {
        $table = $this->table($table);
        $where = $this->where($where);
        $where = !empty($where) ? "WHERE {$where}" : '';

        $result = $this->query("SELECT COUNT(*) FROM {$table} {$where}");

        return $this->result($result);
    }

    /**
     * Get a result row as an enumerated array
     *
     * @param resource $result Resource result comes from query
     * @return mixed
     */
    function fetchRow($result) {
        return mysql_fetch_row($result);
    }

    /**
     * Fetch a result row as an associative array
     *
     * @param resource $result Resource result comes from query
     * @return mixed
     */
    function fetchAssoc($result) {
        return mysql_fetch_assoc($result);
    }

    /**
     * Fetch a result row as an object
     *
     * @param resource $result Resource result comes from query
     * @return mixed
     */
    function fetchObject($result) {
        return mysql_fetch_object($result);
    }

    /**
     * Fetch a result row as an associative array, a numeric array, or both
     *
     * @param resource $result Resource result comes from query
     * @param int $result_type The type of array that is to be fetched (MYSQL_ASSOC, MYSQL_NUM, MYSQL_BOTH)
     * @return mixed
     */
    function fetchArray($result, $result_type = MYSQL_ASSOC) {
        return mysql_fetch_array($result, $result_type);
    }

    /**
     * Check if table exists in database
     *
     * @param string $table Table
     * @return bool
     */
    function tableExists($table = '') {
        $table = $this->table($table);

        return (bool) $this->fetchObject($this->query("SHOW TABLES LIKE '" . $this->escape($table) . "'"));
    }

    /**
     * Check if column exists in table
     *
     * @param string $table Table
     * @param string $column Table column
     * @return bool
     */
    function columnExists($table = '', $column = '') {
        $table = $this->table($table);

        $sql = "SHOW COLUMNS FROM `" . $this->escape($table) . "` LIKE '" . $this->escape($column) . "'";

        return (bool) $this->fetchObject($this->query($sql));
    }

    /**
     * Show debug message
     *
     * @param string $msg Message
     * @param const $error_type Error type
     */
    function debugMessage($msg = '', $error_type = E_USER_ERROR) {
        if ($this->link_id > 0) {
            $this->error = mysql_error($this->link_id);
            $this->errno = mysql_errno($this->link_id);
        } else {
            $this->error = mysql_error();
            $this->errno = mysql_errno();
        }

        $error_msg = empty($msg) ? $this->errno . ': ' . $this->error : $msg;

        if (V_DEBUG > 0) {
            trigger_error($error_msg, $error_type);
        } else {
            $error_msg = $error_msg . ' - ' . date("Y-m-d H:i:s (T)") . "\r\n";
            error_log($error_msg, 3, V_COMPILE_DIR . 'log' . V_DS . 'error.log');

            echo 'Enable DEBUG mode to see error or check error.log file';
        }

        exit;
    }

    /**
     * Renames table
     *
     * @param string $table Table
     * @param string $new_name New table name
     * @return bool
     */
    function renameTable($table, $new_name) {
        $table = $this->table($table);
        $new_name = $this->table($new_name);

        return $this->query("ALTER TABLE `{$table}` RENAME TO `{$new_name}`");
    }

    /**
     * Drops column(s)
     *
     * @param mixed $tables Table(s)
     * @return bool
     */
    function dropColumn($table = '', $columns = null) {
        $table = $this->table($table);

        if (!is_array($columns)) {
            $columns = array($columns);
        }

        return $this->query("ALTER TABLE `" . $table . "` DROP `" . implode("`, DROP `", $columns) . "`");
    }

    /**
     * Drops table(s)
     *
     * @param mixed $tables Table(s)
     * @return bool
     */
    function dropTable($tables) {
        if (!is_array($tables)) {
            $tables = array($tables);
        }

        $t = array();

        foreach ($tables as $k => $table) {
            $t[] = $this->table($table);
        }

        return $this->query("DROP TABLE IF EXISTS `" . implode("`,`", $t) . "`");
    }

    /**
     * Copy table
     *
     * @param string $table Table
     * @param string $new_table New table
     * @return mixed
     */
    function copyTable($table, $new_table) {
        $result = $this->copyTableStructure($table, $new_table);

        if ($result) {
            $table = $this->table($table);
            $new_table = $this->table($new_table);

            $sql = "INSERT INTO `" . $new_table . "` SELECT * FROM " . $table;

            $result = $this->query($sql);
        }

        return $result;
    }

    /**
     * Copy table structure
     *
     * @param string $table Table
     * @param string $new_table New table
     * @return mixed
     */
    function copyTableStructure($table, $new_table) {
        $table = $this->table($table);
        $new_table = $this->table($new_table);

        $sql = "CREATE TABLE IF NOT EXISTS `" . $new_table . "` LIKE " . $table;
        $result = $this->query($sql);

        return $result;
    }

}

?>