<?php

/**
 * @package Vesthelm Engine
 * @version 1.0.0
 * @copyright Copyright(c) 2010-2011, Anton Kurbanov
 * @license Open Source. For more details, please visit: http://www.vesthelm.com/license.html
 * @info Package generated specially for http://test.ir!
 */
defined('V_MAIN') or die('Powered by <a href="http://www.vesthelm.com/">Vesthelm EE</a>');

class NTree extends Vesthelm {

    /**
     * Table
     *
     * @var string
     */
    var $vTable = '';
    /**
     * Primary column name in nested table.
     *
     * @var string
     */
    var $primary_column;
    /**
     * Left column name in nested table.
     *
     * @var string
     */
    var $left_column;
    /**
     * Right column name in nested table.
     *
     * @var string
     */
    var $right_column;
    /**
     * Level column name in nested table.
     *
     * @var string
     */
    var $level_column;

    /**
     * Nested tree constructor
     *
     * @param string $table Table
     * @param string $primary_column Primary column
     * @param string $left_column Left column
     * @param string $right_column Right column
     * @param string $level_column Level column
     */
    function NTree($table = '', $primary_column = 'id', $left_column = 'lft', $right_column = 'rgt', $level_column = 'level') {
        parent::Vesthelm();

        $this->vTable = $this->escape($table);
        $this->primary_column = $this->escape($primary_column);
        $this->left_column = $this->escape($left_column);
        $this->right_column = $this->escape($right_column);
        $this->level_column = $this->escape($level_column);

        if (empty($this->vTable))
            $this->debugMessage('You must supply name of your table in database');

        if (empty($this->primary_column))
            $this->debugMessage('You must supply primary column name');

        if (empty($this->left_column))
            $this->debugMessage('"left" column name must be supplied');

        if (empty($this->right_column))
            $this->debugMessage('"right" column name must be supplied');

        if (empty($this->level_column))
            $this->debugMessage('"level" column name must be supplied');
    }

    /**
     * Get nested tree
     *
     * @param string $fields Fields
     * @param mixed $where WHERE clause
     * @return array
     */
    function getNestedTree($fields, $where = null) {
        $sql = "SELECT {$fields}, `node`.`level` ";
        $sql .= "FROM `" . $this->vPrefix . $this->vTable . "` `node`, `" . $this->vPrefix . $this->vTable . "` `parent` ";
        $sql .= "WHERE `node`.`{$this->left_column}` BETWEEN `parent`.`{$this->left_column}` AND `parent`.`{$this->right_column}` ";
        $sql .= ! empty($where) ? "AND " . $this->where($where) : '';
        $sql .= "GROUP BY `node`.`{$this->primary_column}` ";
        $sql .= "ORDER BY `node`.`{$this->left_column}`";

        return $this->sql($sql);
    }

    /**
     * Edit tree node
     *
     * @param int $id Fields
     * @param array $data Data
     * @param mixed $where WHERE clause
     * @return bool
     */
    function editNode($id, $data, $where = null) {
        $where = !empty($where) ? ' AND ' . $this->where($where) : '';

        $result = $this->update('', $data, "`{$this->primary_column}` = '" . $this->escape($id) . "'" . $where);

        return $result;
    }

    /**
     * Get node by id
     *
     * @param mixed $id Node id
     * @param mixed $where WHERE clause
     * @return array
     */
    function getNode($id, $where = null) {
        $node = array();
        $where = !empty($where) ? ' AND ' . $this->where($where) : '';

        $result = $this->row('', //table
                        "`{$this->left_column}`, `{$this->right_column}`, `{$this->level_column}`", //fields
                        "`{$this->primary_column}` = '" . $this->escape($id) . "'" . $where); //where

        $node[$this->left_column] = (int) $result[$this->left_column];
        $node[$this->right_column] = (int) $result[$this->right_column];
        $node[$this->level_column] = (int) $result[$this->level_column];

        if (empty($node)) {
            $this->debugMessage('Empty node');
        }

        return $node;
    }

    /**
     * Check if root node exists
     *
     * @param mixed $where WHERE clause
     * @return bool
     */
    function rootExists($where = null) {
        $where = !empty($where) ? ' AND ' . $this->where($where) : '';

        return $this->exists('', "`level` = '0'" . $where);
    }

    /**
     * Get root node
     *
     * @param mixed $where WHERE clause
     * @return mixed
     */
    function getRoot($where = null) {
        $where = !empty($where) ? ' AND ' . $this->where($where) : '';

        $root = $this->row('', "*", "`level` = '0'" . $where);

        return $root;
    }

    /**
     * Add root node
     *
     * @param array $data Data
     * @return mixed
     */
    function addRoot($data = array()) {
        $r_data = array();
        $r_data[$this->left_column] = 1;
        $r_data[$this->right_column] = 2;
        $r_data[$this->level_column] = 0;

        $r_data = array_merge($data, $r_data);

        $result = $this->insert('', $r_data); //insert root

        return $result;
    }

    /**
     * Add node
     *
     * @param int $parent_id Parent node id
     * @param array $data Data
     * @param mixed $where WHERE clause
     * @return mixed
     */
    function addNode($parent_id = 0, $data = array(), $where = null) {
        if (0 == $parent_id) {
            return $this->addRoot($data);
        }

        $node_data = array();
        $where = !empty($where) ? ' AND ' . $this->where($where) : '';

        $node = $this->getNode($parent_id);

        $lft = $node[$this->left_column];
        $rgt = $node[$this->right_column];
        $lvl = $node[$this->level_column];

        $node_data[$this->left_column] = $rgt;
        $node_data[$this->right_column] = ($rgt + 1);
        $node_data[$this->level_column] = ($lvl + 1);

        if ($parent_id) {
            $this->update('', //table
                    array(), //fields
                    "`{$this->right_column}` >= {$rgt}" . $where, //where
                    array($this->left_column => "IF (`{$this->left_column}` > {$rgt}, `{$this->left_column}` + 2 , `{$this->left_column}`)",
                        $this->right_column => "IF (`{$this->right_column}` >= {$rgt}, `{$this->right_column}` + 2 , `{$this->right_column}`)"
            ));
        }

        if (empty($data) || empty($node_data)) {
            $this->debugMessage('Empty node data');
        }

        $data = array_merge($data, $node_data);

        return $this->insert('', $data);
    }

    /**
     * Move node
     *
     * @param int $id Node id
     * @param int $parent_id Parent node id
     * @param mixed $where WHERE clause
     * @return mixed
     */
    function moveNode($id, $parent_id = 0, $where = null) {
        $where = !empty($where) ? ' AND ' . $this->where($where) : '';

        $node = $this->getNode($id);

        $lft = $node[$this->left_column];
        $rgt = $node[$this->right_column];
        $lvl = $node[$this->level_column];

        $p_node = $this->getNode($parent_id);

        $p_lft = $p_node[$this->left_column];
        $p_rgt = $p_node[$this->right_column];
        $p_lvl = $p_node[$this->level_column];

        if ($id == $parent_id)
            $this->debugMessage('Node id equal parent node id');

        if ($lft == $p_lft)
            $this->debugMessage('Node left value equal parent node left value');

        if ($p_lft >= $lft && $p_lft <= $rgt) {
            $this->debugMessage('Incorrect left, right, parent left, parent right node values');
        }

        if ($lvl == $p_lvl + 1 && $lft > $p_lft && $rgt < $p_rgt) {
            $this->debugMessage('Incorrect node values');
        }

        // whether it is being moved upwards along the path
        if ($p_lft < $lft && $p_rgt > $rgt && $p_lvl < $lvl - 1) {
            $result = $this->update('', //table
                            array(), //fields
                            "(`{$this->left_column}` BETWEEN " . ($p_lft + 1) . " AND " . ($p_rgt - 1) . ")" . $where, //where

                            array($this->level_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->level_column}`" . sprintf('%+d', -($lvl - 1) + $p_lvl) . ", `{$this->level_column}`)", //additional
                                $this->right_column => "IF (`{$this->right_column}` BETWEEN " . ($rgt + 1) . " AND " . ($p_rgt - 1) . ", `{$this->right_column}` - " . ($rgt - $lft + 1) . ", " .
                                "IF(`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->right_column}` + " . ((($p_rgt - $rgt - $lvl + $p_lvl) / 2) * 2 + $lvl - $p_lvl - 1) . ", `{$this->right_column}`))",
                                $this->left_column => "IF (`{$this->left_column}` BETWEEN " . ($rgt + 1) . " AND " . ($p_rgt - 1) . ", `{$this->left_column}` - " . ($rgt - $lft + 1) . ", " .
                                "IF(`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->left_column}` + " . ((($p_rgt - $rgt - $lvl + $p_lvl) / 2) * 2 + $lvl - $p_lvl - 1) . ", `{$this->left_column}`))"
                    ));
        } elseif ($p_lft < $lft) {
            $result = $this->update('', //table
                            array(), //fields
                            "(`{$this->left_column}` BETWEEN {$p_lft} AND {$rgt} OR `{$this->right_column}` BETWEEN {$p_lft} AND {$rgt})" . $where, //where

                            array($this->level_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->level_column}`" . sprintf('%+d', -($lvl - 1) + $p_lvl) . ", `{$this->level_column}`)", //additional

                                $this->left_column => "IF (`{$this->left_column}` BETWEEN {$p_rgt} AND " . ($lft - 1) . ", `{$this->left_column}` + " . ($rgt - $lft + 1) . ", " .
                                "IF(`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->left_column}` - " . ($lft - $p_rgt) . ", `{$this->left_column}`))",
                                $this->right_column => "IF (`{$this->right_column}` BETWEEN {$p_rgt} AND {$lft}, `{$this->right_column}` + " . ($rgt - $lft + 1) . ", " .
                                "IF(`{$this->right_column}` BETWEEN {$lft} AND {$rgt}, `{$this->right_column}` - " . ($lft - $p_rgt) . ", `{$this->right_column}`))"
                    ));
        } else {
            $result = $this->update('', //table
                            array(), //fields
                            "(`{$this->left_column}` BETWEEN {$lft} AND {$p_rgt} OR `{$this->right_column}` BETWEEN {$lft} AND {$p_rgt})" . $where, //where

                            array($this->level_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->level_column}`" . sprintf('%+d', -($lvl - 1) + $p_lvl) . ", `{$this->level_column}`)", //additional

                                $this->left_column => "IF (`{$this->left_column}` BETWEEN {$rgt} AND {$p_rgt}, `{$this->left_column}` - " . ($rgt - $lft + 1) . ", " .
                                "IF(`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->left_column}` + " . ($p_rgt - 1 - $rgt) . ", `{$this->left_column}`))",
                                $this->right_column => "IF (`{$this->right_column}` BETWEEN " . ($rgt + 1) . " AND " . ($p_rgt - 1) . ", `{$this->right_column}` - " . ($rgt - $lft + 1) . ", " .
                                "IF(`{$this->right_column}` BETWEEN {$lft} AND {$rgt}, `{$this->right_column}` + " . ($p_rgt - 1 - $rgt) . ", `{$this->right_column}`))"
                    ));
        }

        return $result;
    }

    /**
     * Change node position
     *
     * @param int $id Node id
     * @param int $id2 Second node id
     * @param string $position Position
     * @param mixed $where WHERE clause
     * @return mixed
     */
    function changeNodePosition($id, $id2, $position = 'next', $where = null) {
        $where = !empty($where) ? ' AND ' . $this->where($where) : '';
        $node = $this->getNode($id);

        $lft = $node[$this->left_column];
        $rgt = $node[$this->right_column];
        $lvl = $node[$this->level_column];

        $node2 = $this->getNode($id2);

        $lft2 = $node2[$this->left_column];
        $rgt2 = $node2[$this->right_column];
        $lvl2 = $node2[$this->level_column];

        if ($lvl <> $lvl2) {
            $this->debugMessage('Nodes are not at the same level');
        }

        switch ($position) {
            case 'prev':
                if ($lft > $lft2) {
                    $result = $this->update('', //table
                                    array(), //fields
                                    "(`{$this->left_column}` BETWEEN {$lft2} AND {$lft} AND {$rgt})" . $where, //where

                                    array($this->right_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->right_column}` - " . ($lft - $lft2) . ", " . //additional
                                        "IF (`{$this->left_column}` BETWEEN {$lft2} AND " . ($lft - 1) . ", `{$this->right_column}` + " . ($rgt - $lft + 1) . ", `{$this->right_column}`))",
                                        $this->left_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->left_column}` - " . ($lft - $lft2) . ", " .
                                        "IF (`{$this->left_column}` BETWEEN {$lft2} AND " . ($lft - 1) . ", `{$this->left_column}` + " . ($rgt - $lft + 1) . ", `{$this->left_column}`))"
                            ));
                } else {
                    $result = $this->update('', //table
                                    array(), //fields
                                    "(`{$this->left_column}` BETWEEN {$lft} AND " . ($lft2 - 1) . ")" . $where, //where

                                    array($this->right_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->right_column}` + " . (($lft2 - $lft) - ($rgt - $lft + 1)) . ", " . //additional
                                        "IF (`{$this->left_column}` BETWEEN " . ($rgt + 1) . " AND " . ($lft2 - 1) . ", `{$this->right_column}` - " . ($rgt - $lft + 1) . ", `{$this->right_column}`))",
                                        $this->left_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->left_column}` + " . (($lft2 - $lft) - ($rgt - $lft + 1)) . ", " .
                                        "IF (`{$this->left_column}` BETWEEN " . ($rgt + 1) . " AND " . ($lft2 - 1) . ", `{$this->left_column}` - " . ($rgt - $lft + 1) . ", `{$this->left_column}`))"
                            ));
                }
                break;

            case 'next':
                if ($lft > $lft2) {
                    $result = $this->update('', //table
                                    array(), //fields
                                    "(`{$this->left_column}` BETWEEN " . ($rgt2 + 1) . " AND {$rgt})" . $where, //where

                                    array($this->right_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->right_column}` - " . ($lft - $lft2 - ($rgt2 - $lft2 + 1)) . ", " . //additional
                                        "IF (`{$this->left_column}` BETWEEN " . ($rgt2 + 1) . " AND " . ($lft - 1) . ", `{$this->right_column}` + " . ($rgt - $lft + 1) . ", `{$this->right_column}`))",
                                        $this->left_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->left_column}` - " . ($lft - $lft2 - ($rgt2 - $lft2 + 1)) . ", " .
                                        "IF (`{$this->left_column}` BETWEEN " . ($rgt2 + 1) . " AND " . ($lft - 1) . ", `{$this->left_column}` + " . ($rgt - $lft + 1) . ", `{$this->left_column}`))"
                            ));
                } else {
                    $result = $this->update('', //table
                                    array(), //fields
                                    "(`{$this->left_column}` BETWEEN {$lft} AND {$rgt2})" . $where, //where

                                    array($this->right_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->right_column}` + " . ($rgt2 - $rgt) . ", " . //additional
                                        "IF (`{$this->left_column}` BETWEEN " . ($rgt + 1) . " AND {$rgt2}, `{$this->right_column}` - " . ($rgt - $lft + 1) . ", `{$this->right_column}`))",
                                        $this->left_column => "IF (`{$this->left_column}` BETWEEN {$lft} AND {$rgt}, `{$this->left_column}` + " . ($rgt2 - $rgt) . ", " .
                                        "IF (`{$this->left_column}` BETWEEN " . ($rgt + 1) . " AND {$rgt2}, `{$this->left_column}` - " . ($rgt - $lft + 1) . ", `{$this->left_column}`))"
                            ));
                }
                break;

            default:
                $this->debugMessage('Invalid position');
        }

        return $result;
    }

    /**
     * Delete node
     *
     * @param int $id Node id
     * @param mixed $where WHERE clause
     * @return mixed
     */
    function deleteNode($id, $where = null) {
        $where = !empty($where) ? ' AND ' . $this->where($where) : '';
        $node = $this->getNode($id);

        $lft = $node[$this->left_column];
        $rgt = $node[$this->right_column];
        $lvl = $node[$this->level_column];

        $this->delete('', //table
                "(`{$this->left_column}` BETWEEN {$lft} AND {$rgt})" . $where); //where

        $delta = ($rgt - $lft) + 1;

        $result = $this->update('', //table
                        array(), //fields
                        "`{$this->right_column}` > {$rgt}" . $where, //where

                        array($this->left_column => "IF (`{$this->left_column}` > {$lft}, `{$this->left_column}` - {$delta}, `{$this->left_column}`)", //additional
                            $this->right_column => "IF (`{$this->right_column}` > {$lft}, `{$this->right_column}` - {$delta}, `{$this->right_column}`)"
                ));

        return $result;
    }

    /**
     * Nested to multi array
     *
     * @param array $nested Nested tree
     * @return array
     */
    function nestedToMulti($nested) {
        $multi = array();
        $level = array();

        if ($nested) {
            foreach ($nested as $key => $arr) {
                if ($arr['level'] == 0) {
                    $multi[$key] = $arr;
                    $level[$arr['level'] + 1] = $key;
                } else {
                    $parent = &$multi;

                    for ($i = 1; $i <= ( $arr['level'] ); $i++) {
                        $parent = &$parent[$level[$i]];
                    }

                    $parent[$key] = $arr;
                    $level[$arr['level'] + 1] = $key;
                }
            }
        }

        return $multi;
    }

}

?>