<?php

/**
 * @package Vesthelm Engine
 * @version 1.0.0
 * @copyright Copyright(c) 2010-2011, Anton Kurbanov
 * @license Open Source. For more details, please visit: http://www.vesthelm.com/license.html
 * @info Package generated specially for http://test.ir!
 */
defined('V_MAIN') or die('Powered by <a href="http://www.vesthelm.com/">Vesthelm EE</a>');

class Utils {

    /**
     * Get new token
     *
     * @param int $length Token length
     * @return string
     */
    function newToken($length = 32) {
        $token = md5(uniqid(rand(), true));

        if ($length > 0) {
            $token = substr($token, 0, $length);
        }

        return $token;
    }

    /**
     * Create token
     *
     * @param array $params Useful for smarty
     * @return string
     */
    function createToken($params = array()) {
        if (isset($params['addon']) && !empty($params['addon'])) {
            $addon = $params['addon'];
        } else {
            $addon = defined('V_ADDON') ? V_ADDON : 'global';
        }

        $_SESSION['vtokens'][$addon][] = $token = Utils::newToken();

        return $token;
    }

    /**
     * Create tokens
     *
     * @param array $params Useful for smarty
     * @return string
     */
    function createTokens($params = array()) {
        $tokens = '';

        if (isset($params['addon']) && !empty($params['addon'])) {
            $addon = $params['addon'];
        } else {
            $addon = defined('V_ADDON') ? V_ADDON : 'global';
        }

        $name = (isset($params['name']) && !empty($params['name'])) ? $params['name'] : 'global';

        if (isset($params['num']) && !empty($params['num']) && $params['num'] > 0 && !empty($addon)) {
            for ($i = 0; $i <= $params['num']; $i++) {
                $_SESSION['vtokens'][$addon][] = $token = Utils::newToken();

                $tokens .= '<input type="hidden" name="' . $name . '_' . $i . '" value="' . $token . '" />';
            }
        }

        return $tokens;
    }

    /**
     * Create action field
     *
     * @param array $params
     * @return string
     */
    function createActionField($params = array()) {
        $name = (isset($params['name']) && !empty($params['name'])) ? $params['name'] : 'vaction';
        $action = (isset($params['action']) && !empty($params['action'])) ? $params['action'] : '';

        $field = '<input type="hidden" value="' . $action . '" name="' . $name . '" />';

        return $field;
    }

    /**
     * Create id field
     *
     * @param array $params
     * @return string
     */
    function createIdField($params = array()) {
        $name = (isset($params['name']) && !empty($params['name'])) ? $params['name'] : 'vaction';
        $id = (isset($params['id']) && !empty($params['id'])) ? $params['id'] : '';

        $field = '<input type="hidden" value="' . (int) $id . '" name="' . $name . '" />';

        return $field;
    }

    /**
     * Checks token
     *
     * @param array $params Useful for smarty
     * @return bool
     */
    function checkToken($params = array()) {
        if (isset($params['addon']) && !empty($params['addon'])) {
            $addon = $params['addon'];
        } else {
            $addon = defined('V_ADDON') ? V_ADDON : 'global';
        }

        return (!empty($_POST) && empty($_POST['token']) || !empty($_POST) && !in_array($_POST['token'], $_SESSION['vtokens'][$addon]));
    }

    /**
     * Delete addon token
     *
     * @param string $addon Addon
     * @return bool
     */
    function deleteToken($addon) {
        unset($_SESSION['vtokens'][$addon]);

        return true;
    }

    /**
     * Delete all tokens
     *
     * @return bool
     */
    function deleteAllTokens() {
        unset($_SESSION['vtokens']);

        return true;
    }

    /**
     * Check referrer
     *
     * @return bool
     */
    function checkReferrer() {
        if (!empty($_SERVER['HTTP_REFERER'])) {
            $vdomain = parse_url(V_URL);
            $rdomain = parse_url($_SERVER['HTTP_REFERER']);

            if ($vdomain['host'] != $rdomain['host']) {
                return true;
            }
        }

        return false;
    }

    /**
     * Check controller(index.php)
     *
     * @return bool
     */
    function checkController() {
        return (strpos($_SERVER['SCRIPT_NAME'], "index.php") === false);
    }

    /**
     * Check ExtJs header
     *
     * @return bool
     */
    function checkExtHeader() {
        return (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && 'XMLHttpRequest' == $_SERVER['HTTP_X_REQUESTED_WITH'] && $_SERVER['HTTP_X_VESTHELMEXTJS'] != 'VExtJS');
    }

    /**
     * Redirects to some page
     *
     * @param string $file File(`file`)
     * @param string $addon Addon(`addon`)
     * @param bool $acp ACP page or frontend
     */
    function redirect($file, $addon = '', $acp = true) {
        if (!empty($addon)) {
            $url = $acp ? V_ACP_URL . 'index.php?file=' . $file . '&addon=' . $addon : V_URL . 'index.php?file=' . $file . '&addon=' . $addon;
        } else {
            $url = $acp ? V_ACP_URL . $file : V_URL . $file;
        }

        if (!headers_sent($filename, $linenum)) {
            header("Location: " . $url);
            exit;
        }
    }

    /**
     * Sets error reporting
     *
     * @return mixed
     */
    function setErrorReporting() {
        if (V_DEBUG > 0) {
            error_reporting(E_ALL ^ E_NOTICE);
            ini_set('display_errors', 1);
        } else {
            error_reporting(0);
            ini_set('display_errors', 0);
            ini_set('log_errors', 1);
            //ini_set('error_log', V_COMPILE_DIR.'log'.V_DS.'error.log');
        }

        $old_error_handler = set_error_handler("Utils::errorHandler");

        return $old_error_handler;
    }

    /**
     * Error handler
     *
     * @param int $errno Level of the error raised
     * @param string $msg Error message
     * @param string $file Filename that the error was raised in
     * @param int $line Line number the error was raised at
     * @return mixed
     */
    function errorHandler($errno, $msg, $file, $line) {
        if (!($errno & error_reporting())) {
            return;
        }

        while (ob_get_level ()) {
            ob_end_clean();
        }

        $err_type = array(
            E_ERROR => "Error",
            E_WARNING => "Warning",
            E_PARSE => "Parsing Error",
            E_NOTICE => "Notice",
            E_CORE_ERROR => "Core Error",
            E_CORE_WARNING => "Core Warning",
            E_COMPILE_ERROR => "Compile Error",
            E_COMPILE_WARNING => "Compile Warning",
            E_USER_ERROR => "User Error",
            E_USER_WARNING => "User Warning",
            E_USER_NOTICE => "User Notice"
                /* E_STRICT          => "Runtime Notice" PHP5 */
        );

        $info = array();

        if (($errno & E_USER_ERROR) && is_array($arr = unserialize($msg))) {
            foreach ($arr as $k => $v) {
                $info[$k] = $v;
            }
        }

        $trace = array();

        if (function_exists('debug_backtrace')) {
            $trace = debug_backtrace();
            array_shift($trace);
        }

        require_once(V_MAIN . 'error.tpl');
        exit;
    }

    /**
     * Un-quotes a quoted string
     *
     * @param mixed $value Value
     * @return mixed
     */
    function strip_slashes($value) {
        $value = is_array($value) ? array_map('Utils::strip_slashes', $value) : stripslashes($value);

        return $value;
    }

    /**
     * Removes magic quotes
     */
    function removeMagicQuotes() {
        if (get_magic_quotes_gpc ()) {
            $_GET = Utils::strip_slashes($_GET);
            $_POST = Utils::strip_slashes($_POST);
            $_COOKIE = Utils::strip_slashes($_COOKIE);
            $_REQUEST = Utils::strip_slashes($_REQUEST);
        }
    }

    /**
     * Unregister globals
     */
    function unregisterGlobals() {
        if (ini_get('register_globals')) {
            $array = array('_SESSION', '_POST', '_GET', '_COOKIE', '_REQUEST', '_SERVER', '_ENV', '_FILES');

            foreach ($array as $value) {
                foreach ($GLOBALS[$value] as $key => $var) {
                    if ($var === $GLOBALS[$key]) {
                        unset($GLOBALS[$key]);
                    }
                }
            }
        }
    }

    /**
     * Makes recursive directory(s)
     *
     * @param mixed $path_name Directory path
     * @param int $mode Mode
     * @return bool
     */
    function mkdir_r($path_name, $mode = 0777) {
        if (!is_array($path_name)) {
            $path_name = array($path_name);
        }

        foreach ($path_name as $dname) {
            $directories = explode(DIRECTORY_SEPARATOR, $dname);
            $dir = '';

            foreach ($directories as $part) {
                $dir .= $part . DIRECTORY_SEPARATOR;

                if (!is_dir($dir) && !mkdir($dir, $mode)) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Removes file
     *
     * @param string $fullpath File path
     * @return bool
     */
    function removeFile($fullpath) {
        if (file_exists($fullpath)) {
            return unlink($fullpath);
        }

        return false;
    }

    /**
     * Removes directory files
     *
     * @param string $directory Dir
     * @param array $excluded_files Do not deletes these files
     * @return array Removed files
     */
    function removeFiles($directory, $excluded_files = array()) {
        $filenames = array();

        if (is_dir($directory)) {
            if ($dh = opendir($directory)) {
                while (($file = readdir($dh)) !== false) {
                    $pfile = $directory . $file;

                    if (is_file($pfile) && !in_array($file, $excluded_files)) {
                        $result = Utils::removeFile($pfile);

                        if ($result) {
                            $filenames[] = $file;
                        }
                    }
                }
                closedir($dh);
            }
        } else {
            trigger_error("Invalid directory " . $file_path, E_USER_ERROR);
        }

        return $filenames;
    }

    /**
     * Escapes special characters in a string for use in an SQL query
     *
     * @return mixed Escaped string or FALSE on error
     */
    function escape($string) {
        return mysql_real_escape_string($string);
    }

}

?>