<?php

/**
 * @package Vesthelm Engine
 * @version 1.0.0
 * @copyright Copyright(c) 2010-2011, Anton Kurbanov
 * @license Open Source. For more details, please visit: http://www.vesthelm.com/license.html
 * @info Package generated specially for http://test.ir!
 */
defined('V_MAIN') or die('Powered by <a href="http://www.vesthelm.com/">Vesthelm EE</a>');

class Validation {

    /**
     * Check for active addon
     *
     * @param string $addon String
     * @return bool
     */
    function isActiveAddon($addon) {
        global $Vesthelm;
        return (bool) array_key_exists($addon, $Vesthelm->vAddons);
    }

    /**
     * Check for valid string
     *
     * @param string $string String
     * @return bool
     */
    function isValid($string) {
        return (bool) preg_match('/^[A-Za-z0-9_-]*$/', $string);
    }

    /**
     * Check for correct email
     *
     * @param string $email Email
     * @return bool
     */
    function isEmail($email) {
        return (bool) preg_match('/^[^0-9][A-z0-9_]+([.][A-z0-9_]+)*[@][A-z0-9_]+([.][A-z0-9_]+)*[.][A-z]{2,4}$/', $email);
    }

    /**
     * Check for correct url
     *
     * @param string $url URL
     * @return bool
     */
    function isUrl($url) {
        $regex = "((https?|ftp)\:\/\/)?"; // SCHEME
        $regex .= "([a-z0-9+!*(),;?&=\$_.-]+(\:[a-z0-9+!*(),;?&=\$_.-]+)?@)?"; // User and Pass
        $regex .= "([a-z0-9-.]*)\.([a-z]{2,3})"; // Host or IP
        $regex .= "(\:[0-9]{2,5})?"; // Port
        $regex .= "(\/([a-z0-9+\$_-]\.?)+)*\/?"; // Path
        $regex .= "(\?[a-z+&\$_.-][a-z0-9;:@&%=+\/\$_.-]*)?"; // GET Query
        $regex .= "(#[a-z_.-][a-z0-9+\$_.-]*)?"; // Anchor

        return (bool) preg_match("/^$regex$/", $url);
    }

    /**
     * Check for alphanumeric character(s)
     *
     * @param string $string String
     * @return bool
     */
    function isAlNum($string) {
        return (bool) preg_match('/^[a-z0-9]*$/i', $string);
    }

    /**
     * Check for alphabetic character(s)
     *
     * @param string $string String
     * @return bool
     */
    function isAl($string) {
        return (bool) preg_match('/[^a-zA-Z]/', $string);
    }

    /**
     * Check for numeric character(s)
     *
     * @param string $string String
     * @return bool
     */
    function isNum($num) {
        return (bool) preg_match('/\D/', $num);
    }

    /**
     * Includes Utf-8 files for using
     */
    function includeUtf8() {
        $path = V_LIBS . 'utf8' . V_DS . 'utf8.php';

        if (file_exists($path)) {
            require_once($path);
        } else {
            trigger_error('File `utf8.php` does not exist', E_USER_ERROR);
        }
    }

    /**
     * Includes Utf-8 utils
     */
    function includeUtf8Util() {
        if (func_num_args() > 0) {
            foreach (func_get_args () as $util) {
                $path = UTF8 . V_DS . 'utils' . V_DS . $util . '.php';

                if (file_exists($path)) {
                    require_once($path);
                } else {
                    trigger_error('File `' . $util . '.php` does not exist', E_USER_ERROR);
                }
            }
        } else {
            trigger_error('No arguments found', E_USER_ERROR);
        }
    }

}

?>