<?php

/**
 * @package Vesthelm Engine
 * @version 1.0.0
 * @copyright Copyright(c) 2010-2011, Anton Kurbanov
 * @license Open Source. For more details, please visit: http://www.vesthelm.com/license.html
 * @info Package generated specially for http://test.ir!
 */
defined('V_MAIN') or die('Powered by <a href="http://www.vesthelm.com/">Vesthelm EE</a>');

class Vesthelm extends Database {

    /**
     * Hooks
     *
     * @var array
     */
    var $vHooks = array();
    /**
     * Writer
     *
     * @var object
     */
    var $vWriter;
    /**
     * Mail
     *
     * @var object
     */
    var $vMail;
    /**
     * Table prefix
     *
     * @var string
     */
    var $vPrefix;
    /**
     * Languages
     *
     * @var array
     */
    var $vLanguages = array();
    /**
     * Active applications
     *
     * @var array
     */
    var $vApplications;
    /**
     * Active addons
     *
     * @var array
     */
    var $vAddons;
    /**
     * Inactive addons
     *
     * @var array
     */
    var $vInactiveAddons;
    //var $vCharsets = array();
    /**
     * Phrases
     *
     * @var array
     */
    var $vPhrases = array();
    /**
     * Realms
     *
     * @var array
     */
    var $vRealms = array();
    /**
     * Realms phrases names
     *
     * @var array
     */
    var $vRealmsPhrasesNames = array();

    /**
     * Vesthelm constructor
     */
    function Vesthelm() {
        $this->vConfig['vdb_host'] = V_DB_HOST;
        $this->vConfig['vdb_user'] = V_DB_USER;
        $this->vConfig['vdb_password'] = V_DB_PASSWORD;
        $this->vConfig['vdb_name'] = V_DB_NAME;
        $this->vPrefix = V_PREFIX;
        $this->connect();

        $Config = &Config::getInstance();
        $this->vConfig = &$Config->vConf;
        $this->vWriter = $Config->vWriter;

        $this->vMail = new Mail();
        $this->vMail->vConfig = $this->vConfig;
        $this->vMail->setup();

        $this->getActiveApplications();

        $this->getAddons();
        $this->getHooks();
        $this->getLanguages();
        //$this->getCharsets();
        $this->getRealms();

        $side = defined('V_IN_ACP') ? 'acp' : 'frontend';
        $this->getLangPhrases($this->vConfig[$side . '_code'], $side);

        $this->vConfig['vdb_host'] = '';
        $this->vConfig['vdb_user'] = '';
        $this->vConfig['vdb_password'] = '';
        $this->vConfig['vdb_name'] = '';
    }

    /**
     * Get phrases
     *
     * @param string $code Language code
     * @param string $side Side
     * @return array
     */
    function getLangPhrases($code = 'en', $side = 'acp') {
        $side = ('acp' == $side) ? 'acp' : 'frontend';

        $this->vPhrases = $this->vWriter->getContents('vlanguage_' . $side . '_' . $code, true);
        $this->checkPhrasesCacheFiles($code, $side);

        return $this->vPhrases;
    }

    /**
     * Check phrases cache files
     *
     * @param string $code Language code
     * @param string $side Side
     * @return bool
     */
    function checkPhrasesCacheFiles($code, $side) {
        $pfile = $this->vWriter->filePath . 'vlanguage_' . $side . '_' . $code;

        if (file_exists($pfile) && file_exists($pfile . '.js') && !empty($this->vPhrases)) {
            if (isset($this->vConfig['phrases_cache_time']) && (int) $this->vConfig['phrases_cache_time'] > 0) {
                $time = (int) $this->vConfig['phrases_cache_time'];
            } else {
                $time = time() - (14 * 24 * 60 * 60); //14 days; 24 hours; 60 mins; 60secs
            }

            if (filemtime($pfile) < $time || filemtime($pfile . '.js') < $time) {
                $this->createPhrasesCacheFiles($code, $side);
            }
        } else {
            $this->createPhrasesCacheFiles($code, $side);
        }

        return true;
    }

    /**
     * Create phrases cache files for some language
     *
     * @param string $code Language code
     * @param string $side Side
     * @return bool
     */
    function createPhrasesCacheFiles($code, $side) {
        $this->includeClass('Json', true);
        $Json = new Services_JSON();

        $this->vPhrases = $this->nametovalue('languages_phrases', '`name`,`value`', "`side` IN ('{$side}','both') AND `code` = '{$code}'");

        $this->vWriter->writeToFile('vlanguage_' . $side . '_' . $code, $this->vPhrases);
        $this->vWriter->writeToFile('vlanguage_' . $side . '_' . $code . '.js', 'vlanguage = ' . $Json->encode($this->vPhrases) . ';');

        return true;
    }

    /**
     * Create phrases cache files for all available languages
     *
     * @return bool
     */
    function createAllPhrasesCacheFiles() {
        if ($this->vLanguages) {
            foreach ($this->vLanguages as $code => $language) {
                $this->createPhrasesCacheFiles($code, 'acp');
                $this->createPhrasesCacheFiles($code, 'frontend');
            }
        }

        return true;
    }

    /**
     * Get available languages
     *
     * @return array
     */
    function getLanguages() {
        $this->vLanguages = $this->nametovalue('languages', "`code`,`title`");

        return $this->vLanguages;
    }

    /**
     * Get realms
     *
     * @return array
     */
    function getRealms() {
        $this->vRealms = $this->all('realms', '*', '', 0, 0, false, false, 'file', '', 'id');

        if (!empty($this->vRealms)) {
            foreach ($this->vRealms as $r) {
                $this->vRealmsPhrasesNames[$r['side']][$r['addon']][$r['application']][$r['file']][$r['action']] = $r['phrase_name'];
            }
        }

        return $this->vRealms;
    }

    /**
     * Get realm phrase name
     *
     * @return array
     */
    function getRealmPhraseName($side = 'acp', $file = '', $action = '', $addon = '', $application = '') {
        $file = !empty($file) ? $file : 'index';

        if (isset($this->vRealmsPhrasesNames[$side][$addon][$application][$file][$action])) {
            return $this->vRealmsPhrasesNames[$side][$addon][$application][$file][$action];
        }

        return false;
    }

    /*
      function getCharsets() {
      $this->vCharsets = $this->nametovalue('languages', "`code`,`charset`");

      return $this->vCharsets;
      }
     */

    function addHook($name = '', $title = '', $value = '', $code = 'php', $status = 'inactive', $addon = '', $side = '', $application = '') {
        $maxorder = $this->one('hooks', "MAX(`order`) + 1");
        $maxorder = empty($maxorder) ? 1 : $maxorder;

        $hooks = array(
            "name" => $name,
            "value" => $value,
            "code" => in_array($code, array('php', 'smarty')) ? $code : 'php',
            "status" => in_array($status, array('active', 'inactive')) ? $status : 'inactive',
            "application" => $application,
            "addon" => $addon,
            "side" => in_array($side, array('acp', 'frontend')) ? $side : 'frontend',
            "u_id" => Utils::newToken(),
            "order" => $maxorder
        );

        $result = $this->insert('hooks', $hooks);

        if ($result) {
            foreach ($this->vLanguages as $code => $lang) {
                if (!$this->exists('languages_phrases', "`name` = 'vhook_title_" . strtolower($name) . "' AND `code` = '{$code}'")) {
                    $this->insert('languages_phrases', array(
                        'name' => 'vhook_title_' . strtolower($name),
                        'value' => !empty($title) ? $title : $name,
                        'side' => 'other',
                        'addon' => $addon,
                        'application' => $application,
                        'code' => $code)
                    );
                } else {
                    $this->update('languages_phrases', array(
                        'value' => !empty($title) ? $title : $name,
                        'side' => 'other',
                        'addon' => $addon,
                        'application' => $application
                            ),
                            "`name` = 'vhook_title_" . strtolower($name) . "' AND `code` = '{$code}'" //where
                    );
                }
            }
        }

        return $result;
    }

    function editHook($name = '', $title = '', $value = '', $code = 'php', $status = 'inactive', $addon = '', $side = '', $id = 0, $application = '') {
        $hooks = array(
            "name" => $name,
            "value" => $value,
            "code" => in_array($code, array('php', 'smarty')) ? $code : 'php',
            "status" => in_array($status, array('active', 'inactive')) ? $status : 'inactive',
            "application" => $application,
            "addon" => $addon,
            "side" => in_array($side, array('acp', 'frontend')) ? $side : 'frontend'
        );

        $result = $this->update('hooks', $hooks, "`id` = '{$id}'");

        return $result;
    }

    function deleteHook($id = 0, $name = '', $addon = '', $application = '') {
        $and = !empty($application) ? "`application` = '{$application}" : "`addon` = '{$addon}";

        $where = !empty($id) ? "`id` = '" . $this->escape($id) . "'" : "`name` = '" . $this->escape($name) . "' AND " . $and . "'";

        $result = $this->delete('hooks', $where);

        /* if ($result) {
          $result = $this->delete('languages_phrases', "`name` = 'vhook_title_" . strtolower($name) . "'");
          } */

        return $result;
    }

    /**
     * Call hook
     *
     * @param string $hookname Hook name
     */
    function callHook($hookname) {
        if (!empty($this->vHooks)) {
            foreach ($this->vHooks as $key => $hook) {
                if (in_array($hookname, $this->vHooks[$key]) && 'php' == $this->vHooks[$key]['code']) {
                    if ('eval' == $this->vConfig['hooks_run_type']) {
                        $r = eval($hook['value']);

                        if (version_compare(PHP_VERSION, '5.2.0', '>=') && $r === false && ( $error = error_get_last() )) {
                            Utils::errorHandler($error['type'], $error['message'], $error['file'], $error['line']);

                            exit;
                        }
                    } else {
                        $hook_file = $hookname . '_' . $hook['u_id'] . '.php';

                        if (file_exists(V_CACHE_DIR . $hook_file)) {
                            if (isset($this->vConfig['hooks_cache_time']) && (int) $this->vConfig['hooks_cache_time'] > 0) {
                                $time = (int) $this->vConfig['hooks_cache_time'];
                            } else {
                                $time = time() - (14 * 24 * 60 * 60); //14 days; 24 hours; 60 mins; 60secs
                            }

                            if (filemtime(V_CACHE_DIR . $hook_file) < $time) {
                                $this->vWriter->writeToFile($hook_file, '<?php ' . $hook['value'] . ' ?>');
                            }
                        } else {
                            $this->vWriter->writeToFile($hook_file, '<?php ' . $hook['value'] . ' ?>');
                        }

                        require(V_CACHE_DIR . $hook_file);
                    }
                }
            }
        }
    }

    function getActiveApplications() {
        $this->vApplications = $this->nametovalue('applications', "`name`,`version`", "`status` = 'active'");

        return $this->vApplications;
    }

    /**
     * Get all addons (also slice addons by statuses)
     *
     * @return array
     */
    function getAddons() {
        $addons = $this->all('addons', "`name`,`version`,`status`");

        $active = array();
        $inactive = array();

        if ($addons) {
            foreach ($addons as $addon) {
                if ('active' == $addon['status']) {
                    $active[$addon['name']] = $addon['version'];
                } else {
                    $inactive[$addon['name']] = $addon['version'];
                }
            }
        }

        $this->vAddons = $active;
        $this->vInactiveAddons = $inactive;

        return $addons;
    }

    /**
     * Get active addons
     *
     * @return array
     */
    function getActiveAddons() {
        $this->vAddons = $this->nametovalue('addons', "`name`,`version`", "`status` = 'active'");

        return $this->vAddons;
    }

    /**
     * Get inactive addons
     *
     * @return array
     */
    function getInactiveAddons() {
        $this->vInactiveAddons = $this->nametovalue('addons', "`name`,`version`", "`status` = 'inactive'");

        return $this->vInactiveAddons;
    }

    /**
     * Get hooks
     *
     * @return array
     */
    function getHooks() {
        $side = defined('V_IN_ACP') ? 'acp' : 'frontend';
        $this->vHooks = $this->all('hooks', '*', "`status` = 'active' AND `side` = '{$side}' ORDER BY `order`");

        return $this->vHooks;
    }

    /**
     * Used to lazy create classes
     *
     * @param string $classes Classes
     * @param bool $acp Class located in ACP or frontend classes folder
     * @param string $addon Class located in some addon
     */
    function factory($classes, $acp = false, $addon = '', $application = '') {
        if (strpos($classes, ',') !== false) {
            $classes = explode(',', $classes);
        } else {
            $class = trim($classes);
        }

        if (is_array($classes)) {
            foreach ($classes as $class) {
                $class = trim($class);

                global $$class;

                if (!is_object($$class)) {
                    $this->includeClass($class, $acp, $addon, $application);
                }

                $$class = new $class();
            }
        } else {
            global $$class;

            if (!is_object($$class)) {
                $this->includeClass($class, $acp, $addon, $application);
            }

            $$class = new $class();
        }
    }

    /**
     * Include some class(s)
     *
     * @param string $classes Class(s)
     * @param bool $acp Class located in ACP or frontend classes folder
     * @param string $addon Class located in some addon
     */
    function includeClass($classes, $acp = false, $addon = '', $application = '') {
        if (empty($classes)) {
            $this->debugMessage("Empty class name");
        }

        if (!empty($application)) {
            $path = $acp ? V_APPLICATIONS . $application . V_DS . 'acp' . V_DS . 'classes' . V_DS : V_APPLICATIONS . $application . V_DS . 'frontend' . V_DS . 'classes' . V_DS;
        } elseif (!empty($addon)) {
            $path = $acp ? V_ADDONS . $addon . V_DS . 'acp' . V_DS . 'classes' . V_DS : V_ADDONS . $addon . V_DS . 'frontend' . V_DS . 'classes' . V_DS;
        } else {
            $path = $acp ? V_ACP_CLASSES : V_FRONTEND_CLASSES;
        }

        if (strpos($classes, ',') !== false) {
            $classes = explode(',', $classes);
        } else {
            $class = $classes;
        }

        if (is_array($classes)) {
            foreach ($classes as $class) {
                $fullpath = $path . trim($class) . '.php';

                if (!file_exists($fullpath)) {
                    $this->debugMessage("Class `{$class}.php` is not found in `{$path}`");
                } else {
                    require_once($fullpath);
                }
            }
        } else {
            $fullpath = $path . trim($class) . '.php';

            if (!file_exists($fullpath)) {
                $this->debugMessage("Class `{$class}.php` is not found in `{$path}`");
            } else {
                require_once($fullpath);
            }
        }
    }

}

?>