<?php

/**
 * @package Vesthelm Engine
 * @version 1.0.0
 * @copyright Copyright(c) 2010-2011, Anton Kurbanov
 * @license Open Source. For more details, please visit: http://www.vesthelm.com/license.html
 * @info Package generated specially for http://test.ir!
 */
defined('V_MAIN') or die('Powered by <a href="http://www.vesthelm.com/">Vesthelm EE</a>');

class Writer {

    /**
     * File name
     *
     * @var string
     */
    var $fileName;
    /**
     * File handle
     *
     * @var resource
     */
    var $fileHandle;
    /**
     * File buffer
     *
     * @var string
     */
    var $fileBuffer;
    /**
     * File buffer size
     *
     * @var string
     */
    var $fileBufferSize;
    /**
     * File path
     *
     * @var string
     */
    var $filePath = V_CACHE_DIR;

    /**
     * Writer constructor
     */
    function Writer() {
        $this->fileName = '';
        $this->fileHandle = null;
        $this->fileBuffer = '';
        $this->fileBufferSize = 1;
        $this->errors = array();

        if (!is_writeable($this->filePath)) {
            trigger_error($this->filePath . " is not writable", E_USER_ERROR);
        }
    }

    /**
     * Get file contents
     *
     * @param string $filename File
     * @return mixed
     */
    function getContents($filename, $serialized = false) {
        $data = false;

        if (file_exists($this->filePath . $filename)) {
            $contents = file_get_contents($this->filePath . $filename);

            if (!empty($contents)) {
                $data = $serialized ? unserialize($contents) : $contents;
            }
        }

        return $data;
    }

    /**
     * Open file
     *
     * @param string $filename File
     * @return bool
     */
    function openFile($filename) {
        $this->fileName = $this->filePath . $filename;

        if ($this->fileName == '')
            trigger_error("Invalid fileName", E_USER_ERROR);

        if ($this->errors)
            return false;

        $this->fileHandle = fopen($this->fileName, 'wb');

        if (!$this->fileHandle) {
            trigger_error("Could not open file " . $this->fileName, E_USER_ERROR);

            $this->fileHandle = null;
            return false;
        }

        if (!flock($this->fileHandle, LOCK_EX)) {
            trigger_error("Could not lock file " . $this->fileName, E_USER_ERROR);

            $this->fileHandle = null;
            return false;
        }

        return true;
    }

    /**
     * Write some data to file
     *
     * @param string $filename File
     * @param mixed $data File
     * @return bool
     */
    function writeToFile($filename, $data) {
        $this->openFile($filename);

        if ($this->fileHandle === null) {
            trigger_error("No file specified", E_USER_ERROR);

            return false;
        }

        $this->fileBuffer .= ( is_array($data) || is_object($data)) ? serialize($data) : $data;

        if (strlen($this->fileBuffer) > $this->fileBufferSize) {
            if (fwrite($this->fileHandle, $this->fileBuffer, strlen($this->fileBuffer)) === false) {
                trigger_error("Could not write to file", E_USER_ERROR);

                return false;
            }

            $this->fileBuffer = '';
        }

        if (!fflush($this->fileHandle))
            trigger_error("Could not flush output to file", E_USER_ERROR);
        if (!flock($this->fileHandle, LOCK_UN))
            trigger_error("Could not unlock file", E_USER_ERROR);
        if (!fclose($this->fileHandle))
            trigger_error("Could not close file", E_USER_ERROR);

        $this->fileHandle = null;

        return true;
    }

    /**
     * Removes file
     *
     * @param string $filename File
     * @return bool
     */
    function removeFile($filename) {
        $fullpath = $this->filePath . $filename;

        if (file_exists($fullpath)) {
            return unlink($fullpath);
        }

        return false;
    }

    /**
     * Removes directory files
     *
     * @param array $excluded_files Do not deletes these files
     * @return array Removed files
     */
    function removeFiles($excluded_files = array()) {
        $filenames = array();

        if (is_dir($this->filePath)) {
            if ($dh = opendir($this->filePath)) {
                while (($file = readdir($dh)) !== false) {
                    $pfile = $this->filePath . $file;

                    if (is_file($pfile) && !in_array($file, $excluded_files)) {
                        $result = $this->removeFile($file);

                        if ($result) {
                            $filenames[] = $file;
                        }
                    }
                }
                closedir($dh);
            }
        } else {
            trigger_error("Invalid directory " . $this->filePath, E_USER_ERROR);
        }

        return $filenames;
    }

}

?>